@@ -33,14 +33,12 @@ struct _BonoboCanvasComponentPrivate {
 	GnomeCanvasItem   *item;
 };
 
-/*
- * Returns the GnomeCanvasItemClass of an object
- */
+#define PARENT_TYPE BONOBO_X_OBJECT_TYPE
+
+/* Returns the GnomeCanvasItemClass of an object */
 #define ICLASS(x) GNOME_CANVAS_ITEM_CLASS ((GTK_OBJECT (x)->klass))
 
-POA_Bonobo_Canvas_Component__vepv bonobo_canvas_component_vepv;
-
-static BonoboObjectClass *gcc_parent_class;
+static GtkObjectClass *gcc_parent_class;
 
 static gboolean
 CORBA_SVP_Segment_to_SVPSeg (Bonobo_Canvas_SVPSegment *seg, ArtSVPSeg *art_seg)
@@ -123,7 +121,7 @@ restore_state (GnomeCanvasItem *item, const Bonobo_Canvas_State *state)
 	for (i = 0; i < 6; i++)
 		affine [i] = state->item_aff [i];
 
-	gnome_canvas_item_affine_absolute (item, affine);
+	gnome_canvas_item_affine_absolute (item->canvas->root, affine);
 	item->canvas->pixels_per_unit = state->pixels_per_unit;
 	item->canvas->scroll_x1 = state->canvas_scroll_x1;
 	item->canvas->scroll_y1 = state->canvas_scroll_y1;
@@ -133,6 +131,49 @@ restore_state (GnomeCanvasItem *item, const Bonobo_Canvas_State *state)
 	GTK_LAYOUT (item->canvas)->yoffset = state->yoffset;
 }
 
+/* This is copied from gnome-canvas.c since it is declared static */
+static void
+invoke_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags)
+{
+	int child_flags;
+	double *child_affine;
+        double i2w[6], w2c[6], i2c[6];
+
+	child_flags = flags;
+	if (!(item->object.flags & GNOME_CANVAS_ITEM_VISIBLE))
+		child_flags &= ~GNOME_CANVAS_UPDATE_IS_VISIBLE;
+
+	/* Apply the child item's transform */
+        gnome_canvas_item_i2w_affine (item, i2w);
+        gnome_canvas_w2c_affine (item->canvas, w2c);
+        art_affine_multiply (i2c, i2w, w2c);
+        child_affine = i2c;
+
+	/* apply object flags to child flags */
+
+	child_flags &= ~GNOME_CANVAS_UPDATE_REQUESTED;
+
+	if (item->object.flags & GNOME_CANVAS_ITEM_NEED_UPDATE)
+		child_flags |= GNOME_CANVAS_UPDATE_REQUESTED;
+
+	if (item->object.flags & GNOME_CANVAS_ITEM_NEED_AFFINE)
+		child_flags |= GNOME_CANVAS_UPDATE_AFFINE;
+
+	if (item->object.flags & GNOME_CANVAS_ITEM_NEED_CLIP)
+		child_flags |= GNOME_CANVAS_UPDATE_CLIP;
+
+	if (item->object.flags & GNOME_CANVAS_ITEM_NEED_VIS)
+		child_flags |= GNOME_CANVAS_UPDATE_VISIBILITY;
+
+	if ((child_flags & (GNOME_CANVAS_UPDATE_REQUESTED
+			    | GNOME_CANVAS_UPDATE_AFFINE
+			    | GNOME_CANVAS_UPDATE_CLIP
+			    | GNOME_CANVAS_UPDATE_VISIBILITY))
+	    && GNOME_CANVAS_ITEM_CLASS (item->object.klass)->update)
+		(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->update) (
+			item, child_affine, clip_path, child_flags);
+}
+
 static Bonobo_Canvas_ArtUTA *
 impl_Bonobo_Canvas_Component_update (PortableServer_Servant     servant,
 				     const Bonobo_Canvas_State *state,
@@ -180,7 +221,7 @@ impl_Bonobo_Canvas_Component_update (PortableServer_Servant     servant,
 		}
 	}
 	
-	ICLASS (item)->update (item, (double *)aff, svp, flags);
+	invoke_update (item, (double *)aff, svp, flags);
 
 	if (svp){
 		for (i = 0; i < svp->n_segs; i++)
@@ -555,44 +596,6 @@ impl_Bonobo_Canvas_Component_setBounds (PortableServer_Servant     servant,
 	gtk_signal_emit (GTK_OBJECT (gcc), gcc_signals [SET_BOUNDS], bbox, &ev);
 }
 
-/**
- * bonobo_canvas_component_get_epv:
- *
- * Returns: The EPV for the BonoboCanvasComponent.
- */
-POA_Bonobo_Canvas_Component__epv *
-bonobo_canvas_component_get_epv (void)
-{
-	POA_Bonobo_Canvas_Component__epv *epv;
-
-	epv = g_new0 (POA_Bonobo_Canvas_Component__epv, 1);
-
-	epv->update         = impl_Bonobo_Canvas_Component_update;
-	epv->realize        = impl_Bonobo_Canvas_Component_realize;
-	epv->unrealize      = impl_Bonobo_Canvas_Component_unrealize;
-	epv->map            = impl_Bonobo_Canvas_Component_map;
-	epv->unmap          = impl_Bonobo_Canvas_Component_unmap;
-	epv->draw           = impl_Bonobo_Canvas_Component_draw;
-	epv->render         = impl_Bonobo_Canvas_Component_render;
-	epv->bounds         = impl_Bonobo_Canvas_Component_bounds;
-	epv->event          = impl_Bonobo_Canvas_Component_event;
-	epv->contains       = impl_Bonobo_Canvas_Component_contains;
-	epv->setCanvasSize  = impl_Bonobo_Canvas_Component_setCanvasSize;
-	epv->setBounds      = impl_Bonobo_Canvas_Component_setBounds;
-
-	return epv;
-}
-
-static void
-gcc_corba_class_init (void)
-{
-	/*
-	 * Initialize the VEPV
-	 */
-	bonobo_canvas_component_vepv.Bonobo_Unknown_epv     = bonobo_object_get_epv ();
-	bonobo_canvas_component_vepv.Bonobo_Canvas_Component_epv = bonobo_canvas_component_get_epv ();
-}
-
 static void
 gcc_destroy (GtkObject *object)
 {
@@ -600,7 +603,7 @@ gcc_destroy (GtkObject *object)
 
 	gtk_object_destroy (GTK_OBJECT (item->canvas));
 
-	GTK_OBJECT_CLASS (gcc_parent_class)->destroy (object);
+	gcc_parent_class->destroy (object);
 }
 
 static void
@@ -610,15 +613,17 @@ gcc_finalize (GtkObject *object)
 
 	g_free (gcc->priv);
 
-	GTK_OBJECT_CLASS (gcc_parent_class)->finalize (object);
+	gcc_parent_class->finalize (object);
 }
 
 static void
-gcc_class_init (GtkObjectClass *object_class)
+gcc_class_init (BonoboCanvasComponentClass *klass)
 {
-	gcc_parent_class = gtk_type_class (bonobo_object_get_type ());
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+	POA_Bonobo_Canvas_Component__epv *epv = &klass->epv;
+
+	gcc_parent_class = gtk_type_class (PARENT_TYPE);
 
-	gcc_corba_class_init ();
 	object_class->destroy  = gcc_destroy;
 	object_class->finalize = gcc_finalize;
 
@@ -638,6 +643,19 @@ gcc_class_init (GtkObjectClass *object_class)
                         GTK_TYPE_BOOL, 1, GTK_TYPE_POINTER);
 
 	gtk_object_class_add_signals (object_class, gcc_signals, LAST_SIGNAL);
+
+	epv->update         = impl_Bonobo_Canvas_Component_update;
+	epv->realize        = impl_Bonobo_Canvas_Component_realize;
+	epv->unrealize      = impl_Bonobo_Canvas_Component_unrealize;
+	epv->map            = impl_Bonobo_Canvas_Component_map;
+	epv->unmap          = impl_Bonobo_Canvas_Component_unmap;
+	epv->draw           = impl_Bonobo_Canvas_Component_draw;
+	epv->render         = impl_Bonobo_Canvas_Component_render;
+	epv->bounds         = impl_Bonobo_Canvas_Component_bounds;
+	epv->event          = impl_Bonobo_Canvas_Component_event;
+	epv->contains       = impl_Bonobo_Canvas_Component_contains;
+	epv->setCanvasSize  = impl_Bonobo_Canvas_Component_setCanvasSize;
+	epv->setBounds      = impl_Bonobo_Canvas_Component_setBounds;
 }
 
 static void
@@ -653,7 +671,7 @@ bonobo_canvas_component_get_type (void)
 {
 	static GtkType type = 0;
 
-	if (!type){
+	if (!type) {
 		GtkTypeInfo info = {
 			"BonoboCanvasComponent",
 			sizeof (BonoboCanvasComponent),
@@ -665,33 +683,16 @@ bonobo_canvas_component_get_type (void)
 			(GtkClassInitFunc) NULL
 		};
 
-		type = gtk_type_unique (bonobo_object_get_type (), &info);
+		type = bonobo_x_type_unique (
+			PARENT_TYPE,
+			POA_Bonobo_Canvas_Component__init, NULL,
+			GTK_STRUCT_OFFSET (BonoboCanvasComponentClass, epv),
+			&info);
 	}
 
 	return type;
 }
 
-Bonobo_Canvas_Component
-bonobo_canvas_component_object_create (BonoboObject *object)
-{
-	POA_Bonobo_Canvas_Component *servant;
-	CORBA_Environment ev;
-	
-	servant = (POA_Bonobo_Canvas_Component *) g_new0 (BonoboObjectServant, 1);
-	servant->vepv = &bonobo_canvas_component_vepv;
-
-	CORBA_exception_init (&ev);
-	POA_Bonobo_Canvas_Component__init ((PortableServer_Servant) servant, &ev);
-	if (BONOBO_EX (&ev)){
-		g_free (servant);
-		CORBA_exception_free (&ev);
-		return CORBA_OBJECT_NIL;
-	}
-
-	CORBA_exception_free (&ev);
-	return (Bonobo_View) bonobo_object_activate_servant (object, servant);
-}
-
 /**
  * bonobo_canvas_component_construct:
  * @comp: a #BonoboCanvasComponent to initialize
@@ -705,16 +706,11 @@ bonobo_canvas_component_object_create (BonoboObject *object)
  */
 BonoboCanvasComponent *
 bonobo_canvas_component_construct (BonoboCanvasComponent  *comp,
-				  Bonobo_Canvas_Component  corba_canvas_comp,
 				  GnomeCanvasItem         *item)
 {
-	g_return_val_if_fail (comp != NULL, NULL);
-	g_return_val_if_fail (BONOBO_IS_CANVAS_COMPONENT (comp), NULL);
-	g_return_val_if_fail (corba_canvas_comp != NULL, NULL);
-	g_return_val_if_fail (item != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_CANVAS_ITEM (item), NULL);
+	g_return_val_if_fail (BONOBO_IS_CANVAS_COMPONENT (comp), NULL);
 
-	bonobo_object_construct (BONOBO_OBJECT (comp), corba_canvas_comp);
 	comp->priv->item = item;
 
 	return comp;
@@ -734,20 +730,12 @@ BonoboCanvasComponent *
 bonobo_canvas_component_new (GnomeCanvasItem *item)
 {
 	BonoboCanvasComponent *comp;
-	Bonobo_Canvas_Component corba_canvas_comp;
 	
-	g_return_val_if_fail (item != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_CANVAS_ITEM (item), NULL);
 	
 	comp = gtk_type_new (bonobo_canvas_component_get_type ());
-	corba_canvas_comp = bonobo_canvas_component_object_create (
-		BONOBO_OBJECT (comp));
 
-	if (corba_canvas_comp == CORBA_OBJECT_NIL){
-		bonobo_object_unref (BONOBO_OBJECT (comp));
-		return NULL;
-	}
-	return bonobo_canvas_component_construct (comp, corba_canvas_comp, item);
+	return bonobo_canvas_component_construct (comp, item);
 }
 
 /** 
@@ -824,18 +812,19 @@ rih_update (GnomeCanvasItem *item, double affine [6], ArtSVP *svp, int flags)
 {
 	RootItemHack *rih = (RootItemHack *) item;
 	CORBA_Environment ev;
-	Bonobo_Canvas_ArtUTA *cuta;
-
-	cuta = CORBA_UTA (item->canvas->redraw_area);
+	GnomeCanvasItemClass *gci_class = gtk_type_class (
+					gnome_canvas_item_get_type ());
 
 	CORBA_exception_init (&ev);
-	Bonobo_Canvas_ComponentProxy_updateArea (rih->proxy, cuta, &ev);
-	CORBA_free (cuta);
+	Bonobo_Canvas_ComponentProxy_requestUpdate (rih->proxy, &ev);
 	CORBA_exception_free (&ev);
 
 	/*
-	 * Mark our canvas as fully updated
+	 * Mark our canvas and item as fully updated
 	 */
+
+	(* gci_class->update) (item, affine, svp, flags);
+
 	if (item->canvas->redraw_area)
 		art_uta_free (item->canvas->redraw_area);
 	item->canvas->redraw_area = NULL;
