@@ -17,9 +17,9 @@
 #include <bonobo/bonobo-running-context.h>
 #include <time.h>
 
-static BonoboObjectClass    *bonobo_event_source_parent_class;
-POA_Bonobo_EventSource__vepv bonobo_event_source_vepv;
+#define PARENT_TYPE BONOBO_X_OBJECT_TYPE
 
+static GtkObjectClass *bonobo_event_source_parent_class;
 
 struct _BonoboEventSourcePrivate {
 	GSList  *listeners;  /* CONTAINS: ListenerDesc* */
@@ -193,34 +193,6 @@ bonobo_event_source_notify_listeners_full (BonoboEventSource *event_source,
 	g_free (event_name);
 }
 
-
-/**
- * bonobo_event_source_get_epv:
- *
- * Returns: The EPV for the default BonoboEventSource implementation.  
- */
-POA_Bonobo_EventSource__epv *
-bonobo_event_source_get_epv (void)
-{
-	POA_Bonobo_EventSource__epv *epv;
-
-	epv = g_new0 (POA_Bonobo_EventSource__epv, 1);
-
-	epv->addListener         = impl_Bonobo_EventSource_addListener;
-	epv->addListenerWithMask = impl_Bonobo_EventSource_addListenerWithMask;
-	epv->removeListener      = impl_Bonobo_EventSource_removeListener;
-
-	return epv;
-}
-
-static void
-init_event_source_corba_class (void)
-{
-	/* The VEPV */
-	bonobo_event_source_vepv.Bonobo_Unknown_epv = bonobo_object_get_epv ();
-	bonobo_event_source_vepv.Bonobo_EventSource_epv = bonobo_event_source_get_epv ();
-}
-
 static void
 bonobo_event_source_destroy (GtkObject *object)
 {
@@ -240,7 +212,7 @@ bonobo_event_source_destroy (GtkObject *object)
 	g_slist_free (event_source->priv->listeners);
 	g_free (event_source->priv);
 
-	GTK_OBJECT_CLASS (bonobo_event_source_parent_class)->destroy (object);
+	bonobo_event_source_parent_class->destroy (object);
 }
 
 static void
@@ -257,13 +229,15 @@ static void
 bonobo_event_source_class_init (BonoboEventSourceClass *klass)
 {
 	GtkObjectClass *oclass = (GtkObjectClass *) klass;
+	POA_Bonobo_EventSource__epv *epv = &klass->epv;
 
-	bonobo_event_source_parent_class = 
-		gtk_type_class (bonobo_object_get_type ());
+	bonobo_event_source_parent_class = gtk_type_class (PARENT_TYPE);
 
 	oclass->destroy = bonobo_event_source_destroy;
 
-	init_event_source_corba_class ();
+	epv->addListener         = impl_Bonobo_EventSource_addListener;
+	epv->addListenerWithMask = impl_Bonobo_EventSource_addListenerWithMask;
+	epv->removeListener      = impl_Bonobo_EventSource_removeListener;
 }
 
 /**
@@ -290,66 +264,17 @@ bonobo_event_source_get_type (void)
                         (GtkClassInitFunc) NULL
                 };
 
-                type = gtk_type_unique (bonobo_object_get_type (), &info);
+                type = bonobo_x_type_unique (
+			PARENT_TYPE,
+			POA_Bonobo_EventSource__init,
+			NULL,
+			GTK_STRUCT_OFFSET (BonoboEventSourceClass, epv),
+			&info);
         }
 
         return type;
 }
 
-/**
- * bonobo_event_source_corba_object_create:
- * @object: the object to tie the CORBA object to
- * 
- * creates the CORBA object associated with an event source
- * 
- * Return value: the CORBA handle to this object.
- **/
-Bonobo_EventSource
-bonobo_event_source_corba_object_create (BonoboObject *object)
-{
-        POA_Bonobo_EventSource *servant;
-        CORBA_Environment ev;
-
-        servant = (POA_Bonobo_EventSource *) g_new0 (BonoboObjectServant, 1);
-        servant->vepv = &bonobo_event_source_vepv;
-
-        CORBA_exception_init (&ev);
-
-        POA_Bonobo_EventSource__init ((PortableServer_Servant) servant, &ev);
-        if (BONOBO_EX (&ev)) {
-                g_free (servant);
-                CORBA_exception_free (&ev);
-                return CORBA_OBJECT_NIL;
-        }
-
-        CORBA_exception_free (&ev);
-
-        return bonobo_object_activate_servant (object, servant);
-}
-
-/**
- * bonobo_event_source_construct:
- * @event_source: 
- * @corba_event_source: 
- * 
- * constructs an event source.
- * 
- * Return value: the constructed event source or NULL on error.
- **/
-BonoboEventSource *
-bonobo_event_source_construct (BonoboEventSource  *event_source, 
-			       Bonobo_EventSource corba_event_source) 
-{
-        g_return_val_if_fail (event_source != NULL, NULL);
-        g_return_val_if_fail (BONOBO_IS_EVENT_SOURCE (event_source), NULL);
-        g_return_val_if_fail (corba_event_source != NULL, NULL);
-
-        bonobo_object_construct (BONOBO_OBJECT (event_source), 
-				 corba_event_source);
-
-        return event_source;
-}
-
 /**
  * bonobo_event_source_new:
  *
@@ -366,20 +291,7 @@ bonobo_event_source_construct (BonoboEventSource  *event_source,
 BonoboEventSource *
 bonobo_event_source_new (void)
 {
-	BonoboEventSource *event_source;
-	Bonobo_EventSource corba_event_source;
-	
-	event_source = gtk_type_new (BONOBO_EVENT_SOURCE_TYPE);
-	corba_event_source = bonobo_event_source_corba_object_create (
-		BONOBO_OBJECT (event_source));
-	
-	if (corba_event_source == CORBA_OBJECT_NIL) {
-		bonobo_object_unref (BONOBO_OBJECT (event_source));
-		return NULL;
-	}
-	
-	return bonobo_event_source_construct (
-		event_source, corba_event_source);
+	return gtk_type_new (BONOBO_EVENT_SOURCE_TYPE);
 }
 
 /**
