@@ -8,7 +8,13 @@
  *
  * Copyright 1999,2000 Helix Code, Inc.
  */
+
+#define _GNU_SOURCE   /* for dladdr */
+
 #include <config.h>
+#ifdef HAVE_DLADDR
+#include <dlfcn.h>
+#endif
 #include <stdio.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmarshal.h>
@@ -16,10 +22,15 @@
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-object.h>
+#include <bonobo/bonobo-shlib-factory.h>
 #include "Bonobo.h"
 #include "bonobo-running-context.h"
 #include "bonobo-object-directory.h"
 
+/* Shared stuff with BonoboMObject */
+extern int  bonobo_object_get_refs (BonoboObject            *object);
+extern void bonobo_object_epv_init (POA_Bonobo_Unknown__epv *epv);
+
 /* Assumptions made: sizeof(POA_interfacename) does not change between interfaces */
 
 POA_Bonobo_Unknown__vepv bonobo_object_vepv;
@@ -61,7 +72,6 @@ struct _BonoboObjectPrivate {
 enum {
 	QUERY_INTERFACE,
 	SYSTEM_EXCEPTION,
-	OBJECT_GONE,
 	LAST_SIGNAL
 };
 
@@ -568,6 +578,14 @@ impl_Bonobo_Unknown_queryInterface (PortableServer_Servant  servant,
 	return CORBA_Object_duplicate (local_interface->corba_objref, ev);
 }
 
+void
+bonobo_object_epv_init (POA_Bonobo_Unknown__epv *epv)
+{
+	epv->ref            = impl_Bonobo_Unknown_ref;
+	epv->unref          = impl_Bonobo_Unknown_unref;
+	epv->queryInterface = impl_Bonobo_Unknown_queryInterface;
+}
+
 /**
  * bonobo_object_get_epv:
  *
@@ -580,9 +598,7 @@ bonobo_object_get_epv (void)
 
 	epv = g_new0 (POA_Bonobo_Unknown__epv, 1);
 
-	epv->ref            = impl_Bonobo_Unknown_ref;
-	epv->unref          = impl_Bonobo_Unknown_unref;
-	epv->queryInterface = impl_Bonobo_Unknown_queryInterface;
+	bonobo_object_epv_init (epv);
 
 	return epv;
 }
@@ -647,14 +663,6 @@ bonobo_object_class_init (BonoboObjectClass *klass)
 				GTK_SIGNAL_OFFSET(BonoboObjectClass,system_exception),
 				gtk_marshal_NONE__POINTER_POINTER,
 				GTK_TYPE_NONE, 2, GTK_TYPE_POINTER, GTK_TYPE_POINTER);
-	bonobo_object_signals [OBJECT_GONE] =
-		gtk_signal_new ("object_gone",
-				GTK_RUN_LAST,
-				object_class->type,
-				GTK_SIGNAL_OFFSET(BonoboObjectClass,object_gone),
-				gtk_marshal_NONE__POINTER,
-				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
-
 
 	gtk_object_class_add_signals (object_class, bonobo_object_signals, LAST_SIGNAL);
 
@@ -799,10 +807,67 @@ bonobo_object_init (void)
 	g_atexit (bonobo_object_shutdown);
 }
 
+#ifdef HAVE_DLADDR
+#ifdef __USE_GNU
+static void
+bonobo_track_shlib_objects (BonoboObject *object, gpointer shlib_id)
+{
+	static GHashTable *factory_hash = NULL;
+	static gpointer bonobo_lib_dli_fbase = NULL;
+
+	gpointer dli_fbase = NULL;
+	Dl_info info;
+	BonoboShlibFactory *factory;
+
+	if (!factory_hash) { 
+
+		factory_hash = g_hash_table_new (NULL, NULL);
+
+		if (dladdr ("SHLIB_ID", &info))
+			bonobo_lib_dli_fbase = info.dli_fbase;
+	}
+	
+	if (dladdr (shlib_id, &info))
+		dli_fbase = info.dli_fbase;
+	
+	if (dli_fbase && bonobo_lib_dli_fbase &&
+	    (dli_fbase != bonobo_lib_dli_fbase)) {
+		
+		if (BONOBO_IS_SHLIB_FACTORY (object)) {
+			if (g_hash_table_lookup (factory_hash, dli_fbase)) {
+				/* two factories in the same shared library */
+				g_hash_table_remove (factory_hash, dli_fbase);
+			}
+			g_hash_table_insert (factory_hash, dli_fbase, object);
+		} else {
+			if ((factory = g_hash_table_lookup (factory_hash, 
+							    dli_fbase))) {
+			       
+				bonobo_shlib_factory_track_object (factory,
+								   object);
+			}
+		}	
+	}
+}
+#endif
+#endif
+
+/* this only works if servant->vepv is not dynamically allocated, which should
+ * be true. 
+ */
+Bonobo_Unknown
+bonobo_object_activate_servant (BonoboObject *object, 
+				void         *servant)
+{
+	gpointer vepv = ((POA_Bonobo_Unknown *)servant)->vepv; 
+	return bonobo_object_activate_servant_full (object, servant, vepv);
+}
+
 /**
- * bonobo_object_activate_servant:
+ * bonobo_object_activate_servant_full:
  * @object: a BonoboObject
  * @servant: The servant to activate.
+ * @shlib_id: an address to identify the shared library
  *
  * This routine activates the @servant which is wrapped inside the
  * @object on the bonobo_poa (which is the default POA).
@@ -810,8 +875,10 @@ bonobo_object_init (void)
  * Returns: The CORBA_Object that is wrapped by @object and whose
  * servant is @servant.  Might return CORBA_OBJECT_NIL on failure.
  */
-CORBA_Object
-bonobo_object_activate_servant (BonoboObject *object, void *servant)
+Bonobo_Unknown
+bonobo_object_activate_servant_full (BonoboObject *object, 
+				     void *servant,
+				     gpointer shlib_id)
 {
 	CORBA_Environment ev;
 	Bonobo_Unknown o;
@@ -819,6 +886,9 @@ bonobo_object_activate_servant (BonoboObject *object, void *servant)
 	g_return_val_if_fail (BONOBO_IS_OBJECT (object), CORBA_OBJECT_NIL);
 	g_return_val_if_fail (servant != NULL, CORBA_OBJECT_NIL);
 
+#ifdef __USE_GNU
+	bonobo_track_shlib_objects (object, shlib_id);
+#endif
 	CORBA_exception_init (&ev);
 
 	CORBA_free (PortableServer_POA_activate_object (
@@ -1003,7 +1073,7 @@ bonobo_object_check_env (BonoboObject *object, CORBA_Object obj, CORBA_Environme
 }
 
 /**
- * gnome_unknown_ping:
+ * bonobo_unknown_ping:
  * @object: a CORBA object reference of type Bonobo::Unknown
  *
  * Pings the object @object using the ref/unref methods from Bonobo::Unknown.
@@ -1012,7 +1082,7 @@ bonobo_object_check_env (BonoboObject *object, CORBA_Object obj, CORBA_Environme
  * Returns: %TRUE if the Bonobo::Unknown @object is alive.
  */
 gboolean
-gnome_unknown_ping (Bonobo_Unknown object)
+bonobo_unknown_ping (Bonobo_Unknown object)
 {
 	CORBA_Environment ev;
 	gboolean alive;
@@ -1093,4 +1163,57 @@ bonobo_object_idle_unref (BonoboObject *object)
 	g_idle_add ((GSourceFunc) idle_unref_fn, object);
 }
 
+static void
+unref_list (GSList *l)
+{
+	for (; l; l = l->next)
+		bonobo_object_unref (l->data);
+}
 
+/**
+ * bonobo_object_list_unref_all:
+ * @list: A list of BonoboObjects *s
+ * 
+ *  This routine unrefs all the objects listed in
+ * the list and then removes them from @list: if
+ * they have not already been so removed.
+ **/
+void
+bonobo_object_list_unref_all (GList **list)
+{
+	GList *l;
+	GSList *unrefs = NULL, *u;
+
+	g_return_if_fail (list != NULL);
+
+	for (l = *list; l; l = l->next) {
+		if (l->data && !BONOBO_IS_OBJECT (l->data))
+			g_warning ("Non object in unref list");
+		else if (l->data)
+			unrefs = g_slist_prepend (unrefs, l->data);
+	}
+
+	unref_list (unrefs);
+
+	for (u = unrefs; u; u = u->next)
+		*list = g_list_remove (*list, u->data);
+
+	g_slist_free (unrefs);
+}
+
+void
+bonobo_object_slist_unref_all (GSList **list)
+{
+	g_warning ("Dummy - cut and paste above");
+}
+
+int
+bonobo_object_get_refs (BonoboObject *object)
+{
+	g_return_val_if_fail (BONOBO_IS_OBJECT (object), -1);
+
+	if (!object->priv || !object->priv->ao)
+		return 0;
+	else
+		return object->priv->ao->ref_count;
+}
