@@ -12,8 +12,10 @@
 #include <gtk/gtkmarshal.h>
 #include <bonobo/bonobo-persist.h>
 
+#define PARENT_TYPE BONOBO_X_OBJECT_TYPE
+
 /* Parent GTK object class */
-static BonoboObjectClass *bonobo_persist_parent_class;
+static GtkObjectClass *bonobo_persist_parent_class;
 
 #define CLASS(o) BONOBO_PERSIST_CLASS(GTK_OBJECT(o)->klass)
 
@@ -32,47 +34,24 @@ impl_Bonobo_Persist_getContentTypes (PortableServer_Servant servant,
 	return CLASS (persist)->get_content_types (persist, ev);
 }
 
-/**
- * bonobo_persist_get_epv:
- *
- * Returns: The EPV for the default BonoboPersist implementation.  
- */
-POA_Bonobo_Persist__epv *
-bonobo_persist_get_epv (void)
-{
-	POA_Bonobo_Persist__epv *epv;
-
-	epv = g_new0 (POA_Bonobo_Persist__epv, 1);
-
-	epv->getContentTypes = impl_Bonobo_Persist_getContentTypes;
-
-	return epv;
-}
-
-static void
-init_persist_corba_class (void)
-{
-}
-
 static void
 bonobo_persist_destroy (GtkObject *object)
 {
-	GTK_OBJECT_CLASS (bonobo_persist_parent_class)->destroy (object);
+	bonobo_persist_parent_class->destroy (object);
 }
 
 static void
 bonobo_persist_class_init (BonoboPersistClass *klass)
 {
 	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+	POA_Bonobo_Persist__epv *epv = &klass->epv;
 
-	bonobo_persist_parent_class = gtk_type_class (bonobo_object_get_type ());
+	bonobo_persist_parent_class = gtk_type_class (PARENT_TYPE);
 
-	/*
-	 * Override and initialize methods
-	 */
+	/* Override and initialize methods */
 	object_class->destroy = bonobo_persist_destroy;
 
-	init_persist_corba_class ();
+	epv->getContentTypes = impl_Bonobo_Persist_getContentTypes;
 }
 
 static void
@@ -80,19 +59,6 @@ bonobo_persist_init (BonoboPersist *persist)
 {
 }
 
-BonoboPersist *
-bonobo_persist_construct (BonoboPersist *persist,
-			  Bonobo_Persist corba_persist)
-{
-	g_return_val_if_fail (persist != NULL, NULL);
-	g_return_val_if_fail (BONOBO_IS_PERSIST (persist), NULL);
-	g_return_val_if_fail (corba_persist != CORBA_OBJECT_NIL, NULL);
-	
-	bonobo_object_construct (BONOBO_OBJECT (persist), corba_persist);
-
-	return persist;
-}
-
 /**
  * bonobo_persist_get_type:
  *
@@ -103,7 +69,7 @@ bonobo_persist_get_type (void)
 {
 	static GtkType type = 0;
 
-	if (!type){
+	if (!type) {
 		GtkTypeInfo info = {
 			"BonoboPersist",
 			sizeof (BonoboPersist),
@@ -115,7 +81,12 @@ bonobo_persist_get_type (void)
 			(GtkClassInitFunc) NULL
 		};
 
-		type = gtk_type_unique (bonobo_object_get_type (), &info);
+		type = bonobo_x_type_unique (
+			PARENT_TYPE,
+			POA_Bonobo_Persist__init,
+			NULL,
+			GTK_STRUCT_OFFSET (BonoboPersistClass, epv),
+			&info);
 	}
 
 	return type;
