@@ -245,7 +245,7 @@ encode_value (BonoboUINode      *parent,
 
 BonoboUINode *
 bonobo_property_bag_xml_encode_any (BonoboUINode      *opt_parent,
-				    CORBA_any         *any,
+				    const CORBA_any   *any,
 				    CORBA_Environment *ev)
 {
 	BonoboUINode *node;
