@@ -16,7 +16,8 @@
 #include <bonobo/bonobo-persist-stream.h>
 #include <bonobo/bonobo-transient.h>
 
-POA_Bonobo_PropertyBag__vepv bonobo_property_bag_vepv;
+#define PARENT_TYPE BONOBO_X_OBJECT_TYPE
+
 static GtkObjectClass *parent_class = NULL;
          
 
@@ -34,10 +35,7 @@ struct _BonoboPropertyBagPrivate {
 };
 
 
-/*
- * BonoboPropertyBag CORBA methods.
- */
-
+/* BonoboPropertyBag CORBA methods.*/
 static void
 bonobo_property_bag_foreach_create_list (gpointer key, 
 					 gpointer value,
@@ -232,8 +230,6 @@ bonobo_property_bag_construct (BonoboPropertyBag   *pb,
 			       BonoboEventSource   *es,
 			       gpointer             user_data)
 {
-	CORBA_Object      corba_pb;
-
 	pb->es              = es;
 	pb->priv->set_prop  = set_prop;
 	pb->priv->get_prop  = get_prop;
@@ -241,14 +237,6 @@ bonobo_property_bag_construct (BonoboPropertyBag   *pb,
 
 	bonobo_object_add_interface (BONOBO_OBJECT (pb), BONOBO_OBJECT (es));
 
-	corba_pb = bonobo_property_bag_create_corba_object (BONOBO_OBJECT (pb));
-	if (corba_pb == CORBA_OBJECT_NIL) {
-		bonobo_object_unref (BONOBO_OBJECT (pb));
-		return NULL;
-	}
-
-	bonobo_object_construct (BONOBO_OBJECT (pb), corba_pb);
-
 	if (!(pb->priv->transient = bonobo_transient_new (NULL, bonobo_property_servant_new, bonobo_property_servant_destroy, pb))) {
 		bonobo_object_unref (BONOBO_OBJECT (pb));
 		return NULL;
@@ -257,29 +245,6 @@ bonobo_property_bag_construct (BonoboPropertyBag   *pb,
 	return pb;
 }
 
-CORBA_Object
-bonobo_property_bag_create_corba_object (BonoboObject *object)
-{
-	POA_Bonobo_PropertyBag *servant;
-	CORBA_Environment      ev;
-
-	servant = (POA_Bonobo_PropertyBag *)g_new0 (BonoboObjectServant, 1);
-	servant->vepv = &bonobo_property_bag_vepv;
-
-	CORBA_exception_init (&ev);
-
-	POA_Bonobo_PropertyBag__init ( (PortableServer_Servant) servant, &ev);
-	if (BONOBO_EX (&ev)){
-		CORBA_exception_free (&ev);
-		g_free (servant);
-		return CORBA_OBJECT_NIL;
-	}
-
-	CORBA_exception_free (&ev);
-
-	return bonobo_object_activate_servant (object, servant);
-}
-
 /**
  * bonobo_property_bag_new_full:
  *
@@ -361,28 +326,6 @@ bonobo_property_bag_destroy (GtkObject *object)
 	parent_class->destroy (object);
 }
 
-/**
- * bonobo_property_bag_get_epv:
- *
- * Returns: The EPV for the default BonoboPropertyBag implementation.
- * You will most likely want to provide your implementation.
- */
-POA_Bonobo_PropertyBag__epv *
-bonobo_property_bag_get_epv (void)
-{
-	POA_Bonobo_PropertyBag__epv *epv;
-
-	epv = g_new0 (POA_Bonobo_PropertyBag__epv, 1);
-
-	epv->getProperties        = impl_Bonobo_PropertyBag_getProperties;
-	epv->getPropertyByName    = impl_Bonobo_PropertyBag_getPropertyByName;
-	epv->getPropertyNames     = impl_Bonobo_PropertyBag_getPropertyNames;
-	epv->getEventSource       = impl_Bonobo_PropertyBag_getEventSource;
-
-	return epv;
-}
-
-
 
 /*
  * BonoboPropertyBag property manipulation API.
@@ -407,6 +350,11 @@ bonobo_property_bag_add_full (BonoboPropertyBag  *pb,
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (type != NULL);
 	g_return_if_fail (g_hash_table_lookup (pb->priv->props, name) == NULL);
+
+	if (flags == 0) { /* Compatibility hack */
+		flags = BONOBO_PROPERTY_READABLE |
+			BONOBO_PROPERTY_WRITEABLE;
+	}
 			    
 	if (((flags & BONOBO_PROPERTY_READABLE)  && !get_prop) ||
 	    ((flags & BONOBO_PROPERTY_WRITEABLE) && !set_prop)) {
@@ -424,14 +372,15 @@ bonobo_property_bag_add_full (BonoboPropertyBag  *pb,
 	prop->name          = g_strdup (name);
 	prop->idx           = idx;
 	prop->type          = type;
-	prop->default_value = default_value;
 	prop->docstring     = g_strdup (docstring);
 	prop->flags         = flags;
-
 	prop->get_prop      = get_prop;
 	prop->set_prop      = set_prop;
 	prop->user_data     = user_data;
 
+	if (default_value)
+		prop->default_value = bonobo_arg_copy (default_value);
+
 	g_hash_table_insert (pb->priv->props, prop->name, prop);
 }
 
@@ -758,7 +707,6 @@ bonobo_property_bag_get_default (BonoboPropertyBag *pb,
 		return bonobo_arg_copy (prop->default_value);
 	else {
 		BonoboArg *a = bonobo_arg_new (prop->type);
-		bonobo_arg_init_default (a);
 		return a;
 	}
 }
@@ -828,27 +776,22 @@ bonobo_property_bag_get_flags (BonoboPropertyBag *pb,
 }
 
 
-/*
- * Class/object initialization functions.
- */
+/* Class/object initialization functions. */
 
 static void
-bonobo_property_bag_init_corba_class (void)
+bonobo_property_bag_class_init (BonoboPropertyBagClass *klass)
 {
-	bonobo_property_bag_vepv.Bonobo_Unknown_epv     = bonobo_object_get_epv ();
-	bonobo_property_bag_vepv.Bonobo_PropertyBag_epv = bonobo_property_bag_get_epv ();
-}
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+	POA_Bonobo_PropertyBag__epv *epv = &klass->epv;
 
-static void
-bonobo_property_bag_class_init (BonoboPropertyBagClass *class)
-{
-	GtkObjectClass *object_class = (GtkObjectClass *) class;
-
-	parent_class = gtk_type_class (BONOBO_OBJECT_TYPE);
+	parent_class = gtk_type_class (PARENT_TYPE);
 
 	object_class->destroy = bonobo_property_bag_destroy;
 
-	bonobo_property_bag_init_corba_class ();
+	epv->getProperties        = impl_Bonobo_PropertyBag_getProperties;
+	epv->getPropertyByName    = impl_Bonobo_PropertyBag_getPropertyByName;
+	epv->getPropertyNames     = impl_Bonobo_PropertyBag_getPropertyNames;
+	epv->getEventSource       = impl_Bonobo_PropertyBag_getEventSource;
 }
 
 static void
@@ -881,7 +824,11 @@ bonobo_property_bag_get_gtk_type (void)
 			(GtkClassInitFunc) NULL
 		};
 
-		type = gtk_type_unique (bonobo_object_get_type (), &info);
+		type = bonobo_x_type_unique (
+			PARENT_TYPE,
+			POA_Bonobo_PropertyBag__init, NULL,
+			GTK_STRUCT_OFFSET (BonoboPropertyBagClass, epv),
+			&info);
 	}
 
 	return type;
