@@ -97,7 +97,8 @@ impl_bonobo_ui_sync_toolbar_state (BonoboUISync     *sync,
 	char *min_width;
 	char *behavior;
 	char **behavior_array;
-	GdkPixbuf    *icon_pixbuf;
+	GdkPixbuf *icon_pixbuf;
+	gboolean priority;
 
 	/* FIXME: to debug control problem */
 	gtk_widget_show (widget);
@@ -118,6 +119,15 @@ impl_bonobo_ui_sync_toolbar_state (BonoboUISync     *sync,
 		g_strfreev (behavior_array);
 	}
 
+	if ((txt = bonobo_ui_engine_get_attr (node, cmd_node, "priority"))) {
+		priority = atoi (txt);
+		bonobo_ui_node_free_string (txt);
+	} else
+		priority = FALSE;
+
+	bonobo_ui_toolbar_item_set_want_label (
+		BONOBO_UI_TOOLBAR_ITEM (widget), priority);
+
 	icon_pixbuf = cmd_get_toolbar_pixbuf (node, cmd_node);
 
 	type  = bonobo_ui_engine_get_attr (node, cmd_node, "type");
@@ -450,6 +460,37 @@ parse_look (const char *look)
 		: BONOBO_UI_TOOLBAR_STYLE_ICONS_ONLY;
 }
 
+static BonoboUIToolbarStyle
+get_look (BonoboUIEngine *engine,
+	  BonoboUINode   *node)
+{
+	char      *txt;
+	BonoboUIToolbarStyle look;
+
+	if ((txt = bonobo_ui_node_get_attr (node, "look")))
+		look = parse_look (txt);
+
+	else {
+		GtkWidget           *widget;
+
+		widget = bonobo_ui_engine_node_get_widget (engine, node);
+
+		if (!widget || !BONOBO_IS_UI_TOOLBAR (widget) ||
+		    bonobo_ui_toolbar_get_orientation (BONOBO_UI_TOOLBAR (widget)) ==
+		    GTK_ORIENTATION_HORIZONTAL) {
+			txt = bonobo_ui_node_get_attr (node, "hlook");
+			look = parse_look (txt);
+			bonobo_ui_node_free_string (txt);
+		} else {
+			txt = bonobo_ui_node_get_attr (node, "vlook");
+			look = parse_look (txt);
+			bonobo_ui_node_free_string (txt);
+		}
+	}		
+	
+	return look;
+}
+
 static char *
 do_config_popup (BonoboUIEngineConfig *config,
 		 BonoboUINode         *config_node,
@@ -465,9 +506,8 @@ do_config_popup (BonoboUIEngineConfig *config,
 		bonobo_ui_node_free_string (txt);
 	}
 
-	style = parse_look (
-		(txt = bonobo_ui_node_get_attr (config_node, "look")));
-	bonobo_ui_node_free_string (txt);
+	style = get_look (bonobo_ui_engine_config_get_engine (config),
+			  config_node);
 
 	txt = g_strdup_printf (
 		"<Root>"
@@ -489,6 +529,8 @@ do_config_popup (BonoboUIEngineConfig *config,
 		"<separator/>"
 		"<menuitem verb=\"Tip\" _label=\"%s\" set=\"%d\"/>"
 		"<menuitem verb=\"Hide\" _label=\"%s\"/>"
+		"<menuitem verb=\"Customize\" _label=\"%s\" _tip=\"%s\""
+		" pixtype=\"stock\" pixname=\"Preferences\"/>"
 		"</popup>"
 		"</popups>"
 		"</Root>",
@@ -499,7 +541,8 @@ do_config_popup (BonoboUIEngineConfig *config,
 		N_("_Icon"),
 		N_("T_ext"),
 		tip ? N_("Hide t_ips") : N_("Show t_ips"),
-		!tip, N_("_Hide toolbar"));
+		!tip, N_("_Hide toolbar"),
+		N_("Customi_ze"), N_("Customize the toolbar"));
 
 	return txt;
 }
@@ -538,8 +581,8 @@ config_verb_fn (BonoboUIEngineConfig *config,
 			else
 				changed = FALSE;
 			
-		} else if (!strcmp (verb, "Configure")) {
-			g_warning ("Do configure");
+		} else if (!strcmp (verb, "Customize")) {
+			bonobo_ui_engine_config_configure (config);
 			changed = FALSE;
 
 		} else
@@ -554,6 +597,15 @@ config_verb_fn (BonoboUIEngineConfig *config,
 		bonobo_ui_engine_config_serialize (config);
 }
 
+static void
+toolbar_style_changed (BonoboUIToolbar *toolbar, gpointer data)
+{
+	GnomeDock *dock;
+
+	dock = (GnomeDock *) data;
+	gtk_widget_queue_resize (GTK_WIDGET (dock));
+}
+
 static GnomeDockItem *
 create_dockitem (BonoboUISyncToolbar *sync,
 		 BonoboUINode        *node,
@@ -606,6 +658,11 @@ create_dockitem (BonoboUISyncToolbar *sync,
 	item = GNOME_DOCK_ITEM (gnome_dock_item_new (
 		dockname, beh));
 
+	if (gnome_preferences_get_toolbar_relief ())
+		gnome_dock_item_set_shadow_type (item, GTK_SHADOW_OUT);
+	else
+		gnome_dock_item_set_shadow_type (item, GTK_SHADOW_NONE);
+
 	gtk_container_set_border_width (GTK_CONTAINER (item), 2);
 
 	if ((prop = bonobo_ui_node_get_attr (node, "placement"))) {
@@ -648,7 +705,10 @@ create_dockitem (BonoboUISyncToolbar *sync,
 
 		
 	toolbar = BONOBO_UI_TOOLBAR (bonobo_ui_toolbar_new ());
-	
+	gtk_signal_connect (GTK_OBJECT (toolbar), "set_style",
+                            GTK_SIGNAL_FUNC (toolbar_style_changed),
+                            sync->dock);
+
 	gtk_container_add (GTK_CONTAINER (item),
 			   GTK_WIDGET (toolbar));
 	gtk_widget_show (GTK_WIDGET (toolbar));
